package jp.ac.osaka_u.ist.sel.m_sano.extractor;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StreamTokenizer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import jp.ac.osaka_u.ist.sel.m_sano.extractor.data.Token;
import jp.ac.osaka_u.ist.sel.m_sano.extractor.data.TokenType;
import jp.ac.osaka_u.ist.sel.y_yuuki.cn.Logger;
import jp.ac.osaka_u.ist.sel.y_yuuki.cn.data.Method;

/**
 * CŋLqꂽ\[XR[h̎͂s.
 * <p>
 * \bh錾̓肪ړÎ,
 *  "O", "l", "̑L" xɂĂȂ.
 * @author m-sano
 */
public class CMethodExtractor {
	/**
	 * ʎq.
	 * @see {@link StreamTokenizer#TT_WORD}
	 */
	public static final int TT_WORD = StreamTokenizer.TT_WORD;

	/**
	 * l.
	 * @see {@link StreamTokenizer#TT_NUMBER}
	 */
	public static final int TT_NUMBER = StreamTokenizer.TT_NUMBER;

	/**
	 * End of File.
	 * @see {@link StreamTokenizer#TT_EOF}
	 */
	public static final int TT_EOF = StreamTokenizer.TT_EOF;

	/**
	 * s. '\n' Ɠ.
	 * @see {@link StreamTokenizer#TT_EOL}
	 */
	public static final int TT_LF = StreamTokenizer.TT_EOL;

	/** hbg. */
	public static final int TT_DOT = '.';

	/** A. */
	public static final int TT_CR = '\r';

	/** ^u. */
	public static final int TT_TAB = '\t';

	/** pXy[X. */
	public static final int TT_SPACE = ' ';

	/** \C89, C99, C11  S44. */
	public static final ArrayList<String> keywords = new ArrayList<String>(Arrays.asList(
		"_Bool",			"char",				"short",		"int",
		"long",				"signed",			"unsigned",		"float",
		"double",			"_Complex",			"_Imaginary",	"struct",
		"union",			"enum",				"volatile",		"const",
		"restrict",			"auto",				"extern",		"static",
		"register",			"typedef",			"void",			"if",
		"else",				"switch",			"case",			"default",
		"for",				"while",			"do",			"goto",
		"continue",			"break",			"return",		"_Atomic",
		"_Thread_local",	"_Alignas",			"alignof",		"inline",
		"sizeof",			"_Static_assert",	"_Generic",		"_Noreturn"
	));

	/** Ώۃ\[XR[h. */
	private File targetFile;

	/**
	 * ͊𐶐.
	 * @param path Ώۃ\[XR[h̃pX.
	 */
	public CMethodExtractor(String path) {
		this.setTargetFile(path);
	}

	/**
	 * ݂̑Ώۃ\[XR[h擾.
	 * @return Ώۃ\[XR[h.
	 */
	public File getTargetFile() {
		return this.targetFile;
	}

	/**
	 * ͑Ώۂ̃\[XR[hw肷.
	 * @param target ͑Ώۂ̃\[XR[h.
	 */
	public void setTargetFile(File target) {
		this.targetFile = target;
	}

	/**
	 * ͑Ώۂ̃\[XR[hw肷.
	 * @param targetPath ͑Ώۂ̃\[XR[h̃pX.
	 */
	public void setTargetFile(String targetPath) {
		this.setTargetFile(new File(targetPath));
	}



	/**
	 * \bh̒os
	 * @return \bhXg
	 * @throws IOException
	 * @throws FileNotFoundException
	 */
	public ArrayList<Method> extractMethod() throws FileNotFoundException, IOException {
		ArrayList<Method> methodList = new ArrayList<Method>();

		LinkedList<Token> tkList = this.tokenizeToToken();
		Iterator<Token> it = tkList.iterator();

		ArrayList<Token> sentence = new ArrayList<Token>();
		Method md = null;

		int blacketCnt = -1;
		int mdStartBlacket = -1;

		int macroLine = -1;

		while(it.hasNext()) {
			Token tk = it.next();

			Logger.writeln("<extractMethod> token " + tk.getToken(), Logger.DEBUG);

			// }ÑXLbv
			if(macroLine > -1) {
				if(macroLine < tk.getLine()) {

					// }Ns
					if(sentence.size() > 0 && sentence.get(sentence.size()-1).getToken().equals("\\")) {
						macroLine++;
						sentence.add(tk);
						continue;
					} else {
						macroLine = -1;
						sentence.clear();
						sentence.add(tk);
					}
				} else {
					sentence.add(tk);
					continue;
				}
			}

			// \bh錾̊Jnǂ𔻒
			if(tk.getToken().equals("{")) {
				int index = -1;

				// Ƀ\bh錾ȂX[
				if(md == null) {
					//  "(" T, ̒Oʎqł邩
					Iterator<Token> itStInd = sentence.iterator();
					int k = 0;
					while(itStInd.hasNext()) {
						if(itStInd.next().getToken().equals("(")) {
							index = k;
							break;
						}
						k++;
					}

					if(index != -1) {

						//System.err.println(tk.getToken());	// eXgp

						if(index > 0 && sentence.get(index-1).getType() == TokenType.IDENTIFIER) {
							// \bh錾Jnꍇ
							md = new Method();
							md.setName(sentence.get(index-1).getToken());
							md.setStartLine(sentence.get(0).getLine());
							md.setStartColumn(sentence.get(0).getColumn());

							// C̓NX̊TÔŃt@Cőp
							md.setModifiedName(targetFile.getName());

							// ̐`
							int i = index+1;
							String paramStr = "(";


							// eXgp
							/*
							Iterator<Token> testit= sentence.iterator();
							while(testit.hasNext()) {
								System.err.print(testit.next().getToken()+ " ");
							}
							System.err.println();
							*/
							// eXgp܂

							Logger.writeln("<CMethodExtractor> targetFile is " + targetFile.toString(), Logger.DEBUG);


							// Ō ")" { (֐|C^΍)
							int lastParenthIndex = 0;
							for(int l = sentence.size()-1; l >= 0; l--) {
								if(sentence.get(l).getToken().equals(")")) {
									lastParenthIndex = l;
									break;
								}
							}

							while(i < lastParenthIndex) {
								Token paramTk = sentence.get(i);

								// ͋l߂
								if(i == index+1) {
									paramStr = paramStr.concat(paramTk.getToken());
								} else {

									// <TYPE> ͋l߂
									if(paramTk.getToken().equals("<")) {
										paramStr = paramStr.concat("<");
										while(!(paramTk = sentence.get(++i)).getToken().equals(">")) {
											paramStr = paramStr.concat(paramTk.getToken());
										}
										paramStr = paramStr.concat(">");

									// On͊Ԃ
									} else if((paramTk.getType() == TokenType.IDENTIFIER || paramTk.getType() == TokenType.RESERVED)
											&& (sentence.get(i-1).getType() == TokenType.IDENTIFIER || sentence.get(i-1).getType() == TokenType.RESERVED
											|| sentence.get(i-1).getToken().equals(">") || sentence.get(i-1).getToken().equals("]"))) {
										paramStr = paramStr.concat(" " + paramTk.getToken());
									} else {
										paramStr = paramStr.concat(paramTk.getToken());
									}
								}
								i++;
							}
							md.setParams(paramStr + ")");

							mdStartBlacket = blacketCnt+1;

							Logger.writeln("<CMethodExtractor> Extract " + md.getModifiedName() + md.getParams(), Logger.DEBUG);
						}
					}
				}
				sentence.clear();
				blacketCnt++;

			// \bh錾̏I
			} else if(tk.getToken().equals("}")) {

				// eXgp

				/*
				sentence.add(tk);
				Iterator<Token> testit= sentence.iterator();
				while(testit.hasNext()) {
					Token x = testit.next();
					System.err.print(x.getToken()+ " ");
					System.err.print(x.getLine()+ " ");
					System.err.print(blacketCnt+ " == ");
					System.err.print(mdStartBlacket+ " ");
				}
				System.err.println();
				*/

				// eXgp܂

				if(blacketCnt == mdStartBlacket) {
					// \bh錾, , 錾I "}"
					if(md != null) {
						md.setEndLine(tk.getLine());
						md.setEndColumn(tk.getColumn());
						methodList.add(md);

						md = null;
					}

					mdStartBlacket = -1;
				}

				sentence.clear();
				blacketCnt--;
			} else if(tk.getToken().equals(";")) {
				sentence.clear();
			} else if(tk.getToken().equals("#")) {
				sentence.clear();
				macroLine = tk.getLine();
				sentence.add(tk);
			} else {
				sentence.add(tk);
			}
		}

		return methodList;
	}



	/**
	 * ݐݒ肳Ă\[XR[h̎͂s.
	 * Xgvf {@link Token}ɒu.
	 * @return ͌(g[NXg).
	 * @throws FileNotFoundException Ώۃt@C݂Ȃ.
	 * @throws IOException o̓G[.
	 */
	private LinkedList<Token> tokenizeToToken() throws FileNotFoundException, IOException {
		InputStreamReader reader = new InputStreamReader(new FileInputStream(this.targetFile),"JISAutoDetect");

		StreamTokenizer tokenizer = new StreamTokenizer(reader);

		this.setupTokenizer(tokenizer);

		LinkedList<Token> tokenList = new LinkedList<Token>();
		int token;

		int line = 1;
		int column = 1;

		int startLine = -1;
		int startColumn = -1;

		String tkName = null;
		TokenType tkType = TokenType.UNKNOWN;

		while((token = tokenizer.nextToken()) != TT_EOF) {

			switch(token) {
			case TT_WORD:
				tkName = tokenizer.sval;
				if(keywords.contains(tkName)) {
					tkType = TokenType.RESERVED;
				} else if(this.isNumber(tkName)) {
					tkType = TokenType.NUMBER;
				} else {
					tkType = TokenType.IDENTIFIER;
				}
				startLine = line;
				startColumn = column;
				column += tkName.length();

				//System.err.println(tkName + ": " + line);
				break;

			case TT_SPACE:
			case TT_TAB:
				//System.err.println("^u: " + line);
				column++;
				break;

			case TT_LF:
				line++;
				column = 1;
				//System.err.println("s: " + line);
				break;

			case TT_CR:
				break;

			case '/':	// Rg̏
				int next = tokenizer.nextToken();

				// 萔ɂȂ悤ɂ
				tokenizer.ordinaryChar('\'');
				tokenizer.ordinaryChar('"');
				column++;
				if(next == '/') {
					// s܂ŃXLbv
					while((token = tokenizer.nextToken()) != TT_LF && token != TT_EOF);
					line++;
					column = 1;

				} else if(next == '*') {
					// '*/' ܂ŃXLbv
					//System.err.println("/*: " + line);
					while((token = tokenizer.nextToken()) != TT_EOF) {
						//System.err.println(token + ": " + line);
						//System.err.println(tokenizer.toString());

						if(token == TT_WORD) {
							column += tokenizer.sval.length();
							//System.err.println(tokenizer.sval + ": " + line);
						} else {
							//System.err.println(String.valueOf(Character.toChars(token)) + ": " + line);
							column++;
						}

						if(token == '*') {
							int ntk = tokenizer.nextToken();
							//System.err.println(String.valueOf(Character.toChars(token)) + ": " + line);
							if(ntk == '/') {
								column++;
								break;
							}
							tokenizer.pushBack();
						}

						if(token == TT_LF) {
							line++;
							column = 1;
						}
					}
					//System.err.println("*/: " + line);
				} else {
					tokenizer.pushBack();
				}
				tokenizer.quoteChar('\'');
				tokenizer.quoteChar('"');
				break;

			default:
				tkName = String.valueOf(Character.toChars(token));
				tkType = TokenType.SYMBOL;
				startLine = line;
				startColumn = column;
				//System.err.println(tkName + ": " + startLine);
				column++;
				break;
			}

			if(tkName != null) {
				Token tk = new Token(tkName, tkType);
				tk.setPositon(startLine, startColumn);
				tokenList.add(tk);

				tkName = null;
			}
		}

		reader.close();

		return tokenList;
	}



	/**
	 * ^ꂽg[Nlg[Nǂׂ.
	 * lg[N, 擪ł悤 TT_WORD Ƃ.
	 * <p>
	 * {@link StreamTokenizer}̐lg[NƂ͈ӖقȂ.
	 * @param token g[N.
	 * @return lg[NȂ{@code true}, ȊOȂ{@false}.
	 */
	private boolean isNumber(String token) {
		String regex = "^[0-9]";
		Pattern pattern = Pattern.compile(regex);
		Matcher matcher = pattern.matcher(token);
		return matcher.find();
	}


	/**
	 * {@link StreamTokenizer}̐ݒ.
	 * @param tokenizer {@link StreamTokenizer}.
	 */
	private void setupTokenizer(StreamTokenizer tokenizer) {
		tokenizer.resetSyntax();
		tokenizer.wordChars('0', '9');
		tokenizer.wordChars('a', 'z');
		tokenizer.wordChars('A', 'Z');
		tokenizer.wordChars('_', '_');

		tokenizer.quoteChar('\'');
		tokenizer.quoteChar('"');

		tokenizer.eolIsSignificant(true);	// sg[NƂĈ
	}
}
