package jp.ac.osaka_u.ist.sel.m_sano.extractor;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StreamTokenizer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import jp.ac.osaka_u.ist.sel.m_sano.extractor.data.Token;
import jp.ac.osaka_u.ist.sel.m_sano.extractor.data.TokenType;
import jp.ac.osaka_u.ist.sel.y_yuuki.cn.Logger;
import jp.ac.osaka_u.ist.sel.y_yuuki.cn.data.Method;



/**
 * JavaŋLqꂽ\[XR[h̎͂s.
 * <p>
 * \bh錾̓肪ړÎ,
 *  "O", "l", "̑L" xɂĂȂ.
 * @author m-sano
 */
public class JavaMethodExtractor {

	/**
	 * ʎq.
	 * @see {@link StreamTokenizer#TT_WORD}
	 */
	public static final int TT_WORD = StreamTokenizer.TT_WORD;

	/**
	 * l.
	 * @see {@link StreamTokenizer#TT_NUMBER}
	 */
	public static final int TT_NUMBER = StreamTokenizer.TT_NUMBER;

	/**
	 * End of File.
	 * @see {@link StreamTokenizer#TT_EOF}
	 */
	public static final int TT_EOF = StreamTokenizer.TT_EOF;

	/**
	 * s. '\n' Ɠ.
	 * @see {@link StreamTokenizer#TT_EOL}
	 */
	public static final int TT_LF = StreamTokenizer.TT_EOL;

	/** hbg. */
	public static final int TT_DOT = '.';

	/** A. */
	public static final int TT_CR = '\r';

	/** ^u. */
	public static final int TT_TAB = '\t';

	/** pXy[X. */
	public static final int TT_SPACE = ' ';

	/** \ꃊXg. Java8. */
	public static final ArrayList<String> keywords = new ArrayList<String>(Arrays.asList(
		"abstract",	"assert",
		"boolean",	"break",
		"byte",		"case",
		"catch",	"char",
		"class",	"const",
		"continue",	"default",
		"do",		"double",
		"else",		"enum",
		"extends",	"final",
		"finally",	"float",
		"for",		"if",
		"goto",		"implements",
		"import",	"instanceof",
		"int",		"interface",
		"long",		"native",
		"new",		"package",
		"private",	"protected",
		"public",	"return",
		"short",	"static",
		"strictfp",	"super",
		"switch",	"synchronized",
		"this",		"throw",
		"throws",	"transient",
		"try",		"void",
		"volatile",	"while"
	));

	/** Ώۃ\[XR[h. */
	private File targetFile;

	/**
	 * ͊𐶐.
	 * @param path Ώۃ\[XR[h̃pX.
	 */
	public JavaMethodExtractor(String path) {
		this.setTargetFile(path);
	}

	/**
	 * ݂̑Ώۃ\[XR[h擾.
	 * @return Ώۃ\[XR[h.
	 */
	public File getTargetFile() {
		return this.targetFile;
	}

	/**
	 * ͑Ώۂ̃\[XR[hw肷.
	 * @param target ͑Ώۂ̃\[XR[h.
	 */
	public void setTargetFile(File target) {
		this.targetFile = target;
	}

	/**
	 * ͑Ώۂ̃\[XR[hw肷.
	 * @param targetPath ͑Ώۂ̃\[XR[h̃pX.
	 */
	public void setTargetFile(String targetPath) {
		this.setTargetFile(new File(targetPath));
	}



	/**
	 * \bh̒os
	 * @return \bhXg
	 * @throws IOException
	 * @throws FileNotFoundException
	 */
	public ArrayList<Method> extractMethod() throws FileNotFoundException, IOException {
		ArrayList<Method> methodList = new ArrayList<Method>();

		LinkedList<Token> tkList = this.tokenizeToToken();
		Iterator<Token> it = tkList.iterator();

		ArrayList<Token> sentence = new ArrayList<Token>();
		Method md = null;
		ArrayList<String> modifier = new ArrayList<String>();
		ArrayList<Integer> classStartBlacket = new ArrayList<Integer>();

		int blacketCnt = -1;
		int mdStartBlacket = -1;

		boolean isStaticInit = false;
		boolean enumMode = false;
		boolean inEnum = false;
		boolean instanceMethod = false;
		boolean isClassInit = false;

		while(it.hasNext()) {
			Token tk = it.next();

			// CqpɃNXۑ
			if(tk.getToken().equals("class") || tk.getToken().equals("interface")) {
				if(it.hasNext()) {
					tk = it.next();
					if(tk.getType() == TokenType.IDENTIFIER) {
						modifier.add(tk.getToken());
						classStartBlacket.add(blacketCnt+1);
						isClassInit = true;
					}
				}
			}

			// NX/C^tF[X擾܂ňȍ~͎̏{Ȃ
			if(!isClassInit) {
				continue;
			}

			// \bh錾̊Jnǂ𔻒
			if(tk.getToken().equals("{")) {
				int index = -1;

				// Ƀ\bh錾ȂX[
				if(md == null && !isStaticInit) {
					//  "(" T, ̒Oʎqł邩
					Iterator<Token> itStInd = sentence.iterator();
					int k = 0;
					boolean annotation = false;

					while(itStInd.hasNext()) {
						String parentk = itStInd.next().getToken();

						// enum^̐錾ꍇ
						// TODO enum̕\bh錾̋ʂȂ, G[ɗ߂Ă.
						// 炭, clonedetectoȑo͂ǂ, Enum񋓂Ȃ̐錾Ɉ|ꍇ,
						// \bh^Cv1N[ƂĈv邩ǂ`FbNƂɂȂ.
						// , Enum̈قȂ񋓎qԂŃ^Cv1N[Ƃǂ悤Ȃ.
						if(parentk.equals("enum")) {
							enumMode = true;
							break;
						}

						// CX^X̃RXgN^ɑ, gLqĂꍇ
						if(parentk.equals("new")) {
							instanceMethod = true;
							break;
						}

						// enum񋓎q̃RXgN^
						if(enumMode && !inEnum) {
							inEnum = true;
							break;
						}

						// Ame[V݂
						if(parentk.equals("@")) {
							k++;

							// Ame[V
							parentk = itStInd.next().getToken();
							k++;

							parentk = itStInd.next().getToken();
							annotation = true;
						}

						if(parentk.equals("(")) {
							if(annotation) {

								// ")" ܂ŃXLbv
								while(!parentk.equals(")") && itStInd.hasNext()) {
									k++;
									parentk = itStInd.next().getToken();
								}
								annotation = false;

								// Ame[V̕ςȂƂ "{" 炱͔̉p
								if(!itStInd.hasNext()) {
									break;
								}
							} else {
								index = k;
								break;
							}
						}

						// ɓBĂȂ, KAme[VÎ͂
						annotation = false;
						k++;
					}

					if(index != -1) {

						//System.err.println(tk.getToken());	// eXgp

						if(index > 0 && sentence.get(index-1).getType() == TokenType.IDENTIFIER) {
							// \bh錾Jnꍇ
							md = new Method();
							md.setName(sentence.get(index-1).getToken());
							md.setStartLine(sentence.get(0).getLine());
							md.setStartColumn(sentence.get(0).getColumn());

							// Cq̏
							String mod = null;
							Iterator<String> itMod = modifier.iterator();
							while(itMod.hasNext()) {
								if(mod == null) {
									mod = itMod.next();
								} else {
									mod = mod.concat("." + itMod.next());
								}
							}
							md.setModifiedName(mod + "." + md.getName());

							// ̐`
							int i = index+1;
							String paramStr = "(";


							// eXgp
							/*
							Iterator<Token> testit= sentence.iterator();
							while(testit.hasNext()) {
								System.err.print(testit.next().getToken()+ " ");
							}
							System.err.println();
							*/
							// eXgp܂

							Logger.writeln("<JavaMethodExtractor> targetFile is " + targetFile.toString(), Logger.DEBUG);


							while(!sentence.get(i).getToken().equals(")")) {
								Token paramTk = sentence.get(i);

								// ͋l߂
								if(i == index+1) {
									paramStr = paramStr.concat(paramTk.getToken());
								} else {

									// <TYPE> ͋l߂
									if(paramTk.getToken().equals("<")) {
										paramStr = paramStr.concat("<");
										while(!(paramTk = sentence.get(++i)).getToken().equals(">")) {
											paramStr = paramStr.concat(paramTk.getToken());
										}
										paramStr = paramStr.concat(">");

									// ϒ ... ƈ̊Ԃ
									} else if(paramStr.endsWith("...")) {
										paramStr = paramStr.concat(" " + paramTk.getToken());

									// On͊Ԃ
									} else if((paramTk.getType() == TokenType.IDENTIFIER || paramTk.getType() == TokenType.RESERVED)
											&& (sentence.get(i-1).getType() == TokenType.IDENTIFIER || sentence.get(i-1).getType() == TokenType.RESERVED
											|| sentence.get(i-1).getToken().equals(">") || sentence.get(i-1).getToken().equals("]"))) {
										paramStr = paramStr.concat(" " + paramTk.getToken());
									} else {
										paramStr = paramStr.concat(paramTk.getToken());
									}
								}
								i++;
							}
							md.setParams(paramStr + ")");

							mdStartBlacket = blacketCnt+1;

							Logger.writeln("<JavaMethodExtractor> Extract " + md.getModifiedName() + md.getParams(), Logger.DEBUG);
						}
					}

				// X^eBbNCjVCU錾
				} if(sentence.size() == 1 && sentence.get(0).getToken().equals("static")) {
					isStaticInit = true;
					mdStartBlacket = blacketCnt+1;
				}
				sentence.clear();
				blacketCnt++;

			// \bh錾̏I
			} else if(tk.getToken().equals("}")) {

				if(blacketCnt == mdStartBlacket) {
					// \bh錾, , 錾I "}"
					if(md != null) {
						md.setEndLine(tk.getLine());
						md.setEndColumn(tk.getColumn());
						methodList.add(md);

						md = null;
					}

					// staticCjVCȔI
					if(isStaticInit) {
						isStaticInit = false;
					}

					mdStartBlacket = -1;
				} else {
					if(instanceMethod) {
						instanceMethod = false;
					}

					if(inEnum) {
						inEnum = false;
					} else if(enumMode) {
						enumMode = false;
					}
				}

				// TuNX̒Eo
				if(classStartBlacket.get(classStartBlacket.size()-1) == blacketCnt) {
					classStartBlacket.remove(classStartBlacket.size()-1);
					modifier.remove(modifier.size()-1);
				}

				sentence.clear();
				blacketCnt--;
			} else if(tk.getToken().equals(";")) {
				sentence.clear();
			} else {
				sentence.add(tk);
			}
		}

		return methodList;
	}



	/**
	 * ݐݒ肳Ă\[XR[h̎͂s.
	 * Xgvf {@link Token}ɒu.
	 * @return ͌(g[NXg).
	 * @throws FileNotFoundException Ώۃt@C݂Ȃ.
	 * @throws IOException o̓G[.
	 */
	private LinkedList<Token> tokenizeToToken() throws FileNotFoundException, IOException {
		InputStreamReader reader = new InputStreamReader(new FileInputStream(this.targetFile),"JISAutoDetect");

		StreamTokenizer tokenizer = new StreamTokenizer(reader);

		this.setupTokenizer(tokenizer);

		LinkedList<Token> tokenList = new LinkedList<Token>();
		int token;

		int line = 1;
		int column = 1;

		int startLine = -1;
		int startColumn = -1;

		String tkName = null;
		TokenType tkType = TokenType.UNKNOWN;

		while((token = tokenizer.nextToken()) != TT_EOF) {
			switch(token) {
			case TT_WORD:
				tkName = tokenizer.sval;
				if(keywords.contains(tkName)) {
					tkType = TokenType.RESERVED;
				} else if(this.isNumber(tkName)) {
					tkType = TokenType.NUMBER;
				} else {
					tkType = TokenType.IDENTIFIER;
				}
				startLine = line;
				startColumn = column;
				column += tkName.length();
				break;

			case TT_SPACE:
			case TT_TAB:
				column++;
				break;

			case TT_LF:
				line++;
				column = 1;
				break;

			case TT_CR:
				break;

			case '/':	// Rg̏
				int next = tokenizer.nextToken();
				tokenizer.ordinaryChar('\'');
				tokenizer.ordinaryChar('"');
				column++;
				if(next == '/') {
					// s܂ŃXLbv
					while((token = tokenizer.nextToken()) != TT_LF && token != TT_EOF);
					line++;
					column = 1;

				} else if(next == '*') {
					// '*/' ܂ŃXLbv
					while((token = tokenizer.nextToken()) != TT_EOF) {

						if(token == TT_WORD) {
							column += tokenizer.sval.length();
						} else {
							column++;
						}

						if(token == '*') {
							int ntk = tokenizer.nextToken();
							if(ntk == '/') {
								column++;
								break;
							}
							tokenizer.pushBack();
						}

						if(token == TT_LF) {
							line++;
							column = 1;
						}
					}
				} else {
					tokenizer.pushBack();
				}
				tokenizer.quoteChar('\'');
				tokenizer.quoteChar('"');
				break;

			default:
				tkName = String.valueOf(Character.toChars(token));
				tkType = TokenType.SYMBOL;
				startLine = line;
				startColumn = column;
				column++;
			}

			if(tkName != null) {
				Token tk = new Token(tkName, tkType);
				tk.setPositon(startLine, startColumn);
				tokenList.add(tk);

				tkName = null;
			}
		}

		reader.close();

		return tokenList;
	}



	/**
	 * ^ꂽg[Nlg[Nǂׂ.
	 * lg[N, 擪ł悤 TT_WORD Ƃ.
	 * <p>
	 * {@link StreamTokenizer}̐lg[NƂ͈ӖقȂ.
	 * @param token g[N.
	 * @return lg[NȂ{@code true}, ȊOȂ{@false}.
	 */
	private boolean isNumber(String token) {
		String regex = "^[0-9]";
		Pattern pattern = Pattern.compile(regex);
		Matcher matcher = pattern.matcher(token);
		return matcher.find();
	}


	/**
	 * {@link StreamTokenizer}̐ݒ.
	 * @param tokenizer {@link StreamTokenizer}.
	 */
	private void setupTokenizer(StreamTokenizer tokenizer) {
		tokenizer.resetSyntax();
		tokenizer.wordChars('0', '9');
		tokenizer.wordChars('a', 'z');
		tokenizer.wordChars('A', 'Z');
		tokenizer.wordChars('_', '_');

		tokenizer.quoteChar('\'');
		tokenizer.quoteChar('"');

		tokenizer.eolIsSignificant(true);	// sg[NƂĈ
	}
}
