package jp.ac.osaka_u.ist.sel.y_yuuki.cn.analyzer;

import static org.junit.Assert.*;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.util.ArrayList;

import jp.ac.osaka_u.ist.sel.y_yuuki.cn.Def;
import jp.ac.osaka_u.ist.sel.y_yuuki.cn.Logger;
import jp.ac.osaka_u.ist.sel.y_yuuki.cn.analyze.CloneDetectorController;
import jp.ac.osaka_u.ist.sel.y_yuuki.cn.data.Clone;
import jp.ac.osaka_u.ist.sel.y_yuuki.cn.data.CloneSet;
import jp.ac.osaka_u.ist.sel.y_yuuki.cn.data.Project;
import jp.ac.osaka_u.ist.sel.y_yuuki.cn.data.SourceFile;

import org.junit.Test;

public class CloneDetectorControllerTest {

	/*
	 * :
	 *   String root: fBNg ݂ / ݂Ȃ
	 *   boolean javaMode: .java / .c, .h o
	 *
	 * Ԃl:
	 *   t@C̑΃pX̃Xg ArrayList<String>
	 *
	 * O:
	 *   IO: (`FbN)
	 */

	/**
	 * s (.java)
	 */
	@Test
	public void getFileListTest01() throws Exception {
		CloneDetectorController cdc = new CloneDetectorController(new Project());

		ArrayList<String> list = cdc.getFileList("testfile\\src", true);

		assertEquals(5, list.size());

		assertEquals("a\\Test002.java", list.get(0));
		assertEquals("b\\c\\Test005.java", list.get(1));
		assertEquals("b\\Test00A.java", list.get(2));
		assertEquals("Program.java", list.get(3));
		assertEquals("Test001.java", list.get(4));
	}

	/**
	 * s (.c)
	 */
	@Test
	public void getFileListTest02() throws Exception {
		CloneDetectorController cdc = new CloneDetectorController(new Project());

		ArrayList<String> list = cdc.getFileList("testfile\\src", false);

		assertEquals(10, list.size());

		assertEquals("a\\Test02.c", list.get(0));
		assertEquals("a\\Test02.h", list.get(1));
		assertEquals("b\\c\\Test04.c", list.get(2));
		assertEquals("b\\c\\Test04.h", list.get(3));
		assertEquals("b\\Test03.c", list.get(4));
		assertEquals("b\\Test03.h", list.get(5));
		assertEquals("Test.c", list.get(6));
		assertEquals("Test.h", list.get(7));
		assertEquals("Test_B.c", list.get(8));
		assertEquals("Test_B.h", list.get(9));
	}



	/*
	 * :
	 *
	 * Ԃl:
	 *   true: ݒ肪
	 *   false: ݒ肪ُ
	 *
	 * O:
	 */

	/**
	 * s (C)
	 */
	@Test
	public void readCloneDataFileTest01() throws Exception {
		Logger.init();

		copyFile(Def.RESULT_TXT, "testfile/result/c-result-new.txt");
		copyFile(Def.RESULT_TXT_OLD, "testfile/result/c-result-old.txt");

		Project p = new Project();
		p.setLang("c");
		File newdir = new File("testfile/C_TestProj_Toppers/sc1");
		File olddir = new File("testfile/C_TestProj_Toppers/sc1-old");
		p.setNewDir(newdir.getAbsolutePath());
		p.setOldDir(olddir.getAbsolutePath());

		CloneDetectorController cdc = new CloneDetectorController(p);

		assertTrue(cdc.readCloneDataFile());

		ArrayList<CloneSet> csList = p.getCloneSetList();
		assertEquals(3, csList.size());

		/*
		for(int i = 0; i < csList.size(); i++) {
			CloneSet cs = csList.get(i);

			for(int j = 0; j < cs.getNewCloneList().size(); j++) {
				System.err.println("NEW: " + cs.getNewCloneList().get(j).getMethodName());
			}
			for(int j = 0; j < cs.getOldCloneList().size(); j++) {
				System.err.println("OLD: " + cs.getOldCloneList().get(j).getMethodName());
			}
		}
		*/

		CloneSet cs;
		ArrayList<Clone> nl;
		ArrayList<Clone> ol;
		Clone c;
		SourceFile sf;

		//----------------------------------------
		cs = csList.get(0);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(0, cs.getId());
		assertEquals(3, nl.size());

		c = nl.get(0);
		assertEquals(0, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "arch\\logtrace\\trace_dump.c");
		assertEquals(sf, c.getFile());
		assertEquals("trace_print_tskenter", c.getMethodName());

		c = nl.get(1);
		assertEquals(1, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "arch\\logtrace\\trace_dump.c");
		assertEquals(sf, c.getFile());
		assertEquals("trace_print_tskleave", c.getMethodName());

		c = nl.get(2);
		assertEquals(2, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "arch\\logtrace\\trace_dump.c");
		assertEquals(sf, c.getFile());
		assertEquals("trace_print_sysenter", c.getMethodName());

		assertEquals(0, cs.getOldId());
		assertEquals(3, ol.size());

		c = ol.get(0);
		assertEquals(0, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "arch\\logtrace\\trace_dump.c");
		assertEquals(sf, c.getFile());
		assertEquals("trace_print_tskenter", c.getMethodName());

		c = ol.get(1);
		assertEquals(1, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "arch\\logtrace\\trace_dump.c");
		assertEquals(sf, c.getFile());
		assertEquals("trace_print_resenter", c.getMethodName());

		c = ol.get(2);
		assertEquals(2, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "arch\\logtrace\\trace_dump.c");
		assertEquals(sf, c.getFile());
		assertEquals("trace_print_resleave", c.getMethodName());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(1);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(1, cs.getId());
		assertEquals(3, nl.size());

		c = nl.get(0);
		assertEquals(3, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "arch\\logtrace\\trace_dump.c");
		assertEquals(sf, c.getFile());
		assertEquals("trace_print_eventleave", c.getMethodName());

		c = nl.get(1);
		assertEquals(4, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "arch\\logtrace\\trace_dump.c");
		assertEquals(sf, c.getFile());
		assertEquals("trace_print_schtblleave", c.getMethodName());

		c = nl.get(2);
		assertEquals(5, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "arch\\logtrace\\trace_dump.c");
		assertEquals(sf, c.getFile());
		assertEquals("trace_print_sysleave", c.getMethodName());

		assertEquals(-1, cs.getOldId());
		assertEquals(0, ol.size());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(2);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(-1, cs.getId());
		assertEquals(0, nl.size());

		assertEquals(1, cs.getOldId());
		assertEquals(2, ol.size());

		c = ol.get(0);
		assertEquals(3, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "arch\\logtrace\\trace_dump.c");
		assertEquals(sf, c.getFile());
		assertEquals("trace_print_almenter", c.getMethodName());

		c = ol.get(1);
		assertEquals(4, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "arch\\logtrace\\trace_dump.c");

		assertEquals(sf, c.getFile());
		assertEquals("trace_print_almleave", c.getMethodName());
		//----------------------------------------
	}


	/**
	 * s (Java)
	 */
	@Test
	public void readCloneDataFileTest02() throws Exception {
		Logger.init();

		copyFile(Def.RESULT_TXT, "testfile/result/java-result-new.txt");
		copyFile(Def.RESULT_TXT_OLD, "testfile/result/java-result-old.txt");

		Project p = new Project();
		p.setLang("java");
		File newdir = new File("testfile/Java_TestProj_Ant/ant");
		File olddir = new File("testfile/Java_TestProj_Ant/ant-old");
		p.setNewDir(newdir.getAbsolutePath());
		p.setOldDir(olddir.getAbsolutePath());

		CloneDetectorController cdc = new CloneDetectorController(p);

		assertTrue(cdc.readCloneDataFile());

		ArrayList<CloneSet> csList = p.getCloneSetList();
		assertEquals(12, csList.size());

		/*
		for(int i = 0; i < csList.size(); i++) {
			CloneSet cs = csList.get(i);

			for(int j = 0; j < cs.getNewCloneList().size(); j++) {
				System.err.println("NEW: " + cs.getNewCloneList().get(j).getFile().getName());
			}
			for(int j = 0; j < cs.getOldCloneList().size(); j++) {
				System.err.println("OLD: " + cs.getOldCloneList().get(j).getFile().getName());
			}
		}
		*/

		CloneSet cs;
		ArrayList<Clone> nl;
		ArrayList<Clone> ol;
		Clone c;
		SourceFile sf;

		//----------------------------------------
		cs = csList.get(0);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(0, cs.getId());
		assertEquals(2, nl.size());

		c = nl.get(0);
		assertEquals(0, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "ComponentHelper.java");
		assertEquals(sf, c.getFile());
		assertEquals("ComponentHelper.checkTaskClass(final Class<?> taskClass)", c.getMethodName());
		assertEquals(364, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(387, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = nl.get(1);
		assertEquals(1, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "Project.java");
		assertEquals(sf, c.getFile());
		assertEquals("Project.checkTaskClass(final Class<?> taskClass)", c.getMethodName());
		assertEquals(984, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(1014, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		assertEquals(0, cs.getOldId());
		assertEquals(2, ol.size());


		c = ol.get(0);
		assertEquals(0, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "ComponentHelper.java");
		assertEquals(sf, c.getFile());
		assertEquals("ComponentHelper.checkTaskClass(final Class<?> taskClass)", c.getMethodName());
		assertEquals(364, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(387, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = ol.get(1);
		assertEquals(1, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "Project.java");
		assertEquals(sf, c.getFile());
		assertEquals("Project.checkTaskClass(final Class<?> taskClass)", c.getMethodName());
		assertEquals(984, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(1014, c.getEndLine());
		assertEquals(5, c.getEndColumn());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(1);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(1, cs.getId());
		assertEquals(2, nl.size());

		c = nl.get(0);
		assertEquals(2, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "DefaultDefinitions.java");
		assertEquals(sf, c.getFile());
		assertEquals("DefaultDefinitions.attributeNamespaceDef(String ns)", c.getMethodName());
		assertEquals(52, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(59, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = nl.get(1);
		assertEquals(3, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "DefaultDefinitions.java");
		assertEquals(sf, c.getFile());
		assertEquals("DefaultDefinitions.componentDef(String ns,String name,String classname)", c.getMethodName());
		assertEquals(67, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(75, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		assertEquals(1, cs.getOldId());
		assertEquals(2, ol.size());

		c = ol.get(0);
		assertEquals(2, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "DefaultDefinitions.java");
		assertEquals(sf, c.getFile());
		assertEquals("DefaultDefinitions.attributeNamespaceDef(String ns)", c.getMethodName());
		assertEquals(52, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(59, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = ol.get(1);
		assertEquals(3, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "DefaultDefinitions.java");
		assertEquals(sf, c.getFile());
		assertEquals("DefaultDefinitions.componentDef(String ns,String name,String classname)", c.getMethodName());
		assertEquals(67, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(75, c.getEndLine());
		assertEquals(5, c.getEndColumn());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(2);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(2, cs.getId());
		assertEquals(2, nl.size());

		c = nl.get(0);
		assertEquals(4, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "DemuxOutputStream.java");
		assertEquals(sf, c.getFile());
		assertEquals("DemuxOutputStream.getBufferInfo()", c.getMethodName());
		assertEquals(98, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(108, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = nl.get(1);
		assertEquals(5, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "DemuxOutputStream.java");
		assertEquals(sf, c.getFile());
		assertEquals("DemuxOutputStream.resetBufferInfo()", c.getMethodName());
		assertEquals(113, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(123, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		assertEquals(2, cs.getOldId());
		assertEquals(2, ol.size());

		c = ol.get(0);
		assertEquals(4, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "DemuxOutputStream.java");
		assertEquals(sf, c.getFile());
		assertEquals("DemuxOutputStream.getBufferInfo()", c.getMethodName());
		assertEquals(98, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(108, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = ol.get(1);
		assertEquals(5, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "util\\LineOrientedOutputStream.java");
		assertEquals(sf, c.getFile());
		assertEquals("LineOrientedOutputStream.write(byte[] b,int off,int len)", c.getMethodName());
		assertEquals(130, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(152, c.getEndLine());
		assertEquals(5, c.getEndColumn());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(3);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(3, cs.getId());
		assertEquals(2, nl.size());

		c = nl.get(0);
		assertEquals(6, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\HeadFilter.java");
		assertEquals(sf, c.getFile());
		assertEquals("HeadFilter.read()", c.getMethodName());
		assertEquals(101, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(125, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = nl.get(1);
		assertEquals(7, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\TailFilter.java");
		assertEquals(sf, c.getFile());
		assertEquals("TailFilter.read()", c.getMethodName());
		assertEquals(104, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(125, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		assertEquals(3, cs.getOldId());
		assertEquals(2, ol.size());

		c = ol.get(0);
		assertEquals(6, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\HeadFilter.java");
		assertEquals(sf, c.getFile());
		assertEquals("HeadFilter.read()", c.getMethodName());
		assertEquals(101, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(125, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = ol.get(1);
		assertEquals(7, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\TailFilter.java");
		assertEquals(sf, c.getFile());
		assertEquals("TailFilter.read()", c.getMethodName());
		assertEquals(104, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(125, c.getEndLine());
		assertEquals(5, c.getEndColumn());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(4);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(4, cs.getId());
		assertEquals(3, nl.size());

		c = nl.get(0);
		assertEquals(8, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\PrefixLines.java");
		assertEquals(sf, c.getFile());
		assertEquals("PrefixLines.initialize()", c.getMethodName());
		assertEquals(152, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(162, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = nl.get(1);
		assertEquals(9, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\SuffixLines.java");
		assertEquals(sf, c.getFile());
		assertEquals("SuffixLines.initialize()", c.getMethodName());
		assertEquals(162, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(172, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = nl.get(2);
		assertEquals(10, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\TabsToSpaces.java");
		assertEquals(sf, c.getFile());
		assertEquals("TabsToSpaces.initialize()", c.getMethodName());
		assertEquals(141, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(153, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		assertEquals(4, cs.getOldId());
		assertEquals(3, ol.size());

		c = ol.get(0);
		assertEquals(8, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\PrefixLines.java");
		assertEquals(sf, c.getFile());
		assertEquals("PrefixLines.initialize()", c.getMethodName());
		assertEquals(152, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(162, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = ol.get(1);
		assertEquals(9, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\SuffixLines.java");
		assertEquals(sf, c.getFile());
		assertEquals("SuffixLines.initialize()", c.getMethodName());
		assertEquals(162, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(172, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = ol.get(2);
		assertEquals(10, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\TabsToSpaces.java");
		assertEquals(sf, c.getFile());
		assertEquals("TabsToSpaces.initialize()", c.getMethodName());
		assertEquals(141, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(153, c.getEndLine());
		assertEquals(5, c.getEndColumn());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(5);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(5, cs.getId());
		assertEquals(2, nl.size());

		c = nl.get(0);
		assertEquals(11, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\TokenFilter.java");
		assertEquals(sf, c.getFile());
		assertEquals("TokenFilter.ReplaceRegex.initialize()", c.getMethodName());
		assertEquals(472, c.getStartLine());
		assertEquals(9, c.getStartColumn());
		assertEquals(488, c.getEndLine());
		assertEquals(9, c.getEndColumn());

		c = nl.get(1);
		assertEquals(12, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\TokenFilter.java");
		assertEquals(sf, c.getFile());
		assertEquals("TokenFilter.ContainsRegex.initialize()", c.getMethodName());
		assertEquals(540, c.getStartLine());
		assertEquals(9, c.getStartColumn());
		assertEquals(556, c.getEndLine());
		assertEquals(9, c.getEndColumn());

		assertEquals(5, cs.getOldId());
		assertEquals(2, ol.size());

		c = ol.get(0);
		assertEquals(11, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\TokenFilter.java");
		assertEquals(sf, c.getFile());
		assertEquals("TokenFilter.ReplaceRegex.initialize()", c.getMethodName());
		assertEquals(472, c.getStartLine());
		assertEquals(9, c.getStartColumn());
		assertEquals(488, c.getEndLine());
		assertEquals(9, c.getEndColumn());

		c = ol.get(1);
		assertEquals(12, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "filters\\TokenFilter.java");
		assertEquals(sf, c.getFile());
		assertEquals("TokenFilter.ContainsRegex.initialize()", c.getMethodName());
		assertEquals(540, c.getStartLine());
		assertEquals(9, c.getStartColumn());
		assertEquals(556, c.getEndLine());
		assertEquals(9, c.getEndColumn());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(6);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(6, cs.getId());
		assertEquals(2, nl.size());

		c = nl.get(0);
		assertEquals(13, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "helper\\ProjectHelperImpl.java");
		assertEquals(sf, c.getFile());
		assertEquals("ProjectHelperImpl.TaskHandler.startElement(String name,AttributeList attrs)", c.getMethodName());
		assertEquals(771, c.getStartLine());
		assertEquals(9, c.getStartColumn());
		assertEquals(779, c.getEndLine());
		assertEquals(9, c.getEndColumn());

		c = nl.get(1);
		assertEquals(14, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "helper\\ProjectHelperImpl.java");
		assertEquals(sf, c.getFile());
		assertEquals("ProjectHelperImpl.NestedElementHandler.startElement(String name,AttributeList attrs)", c.getMethodName());
		assertEquals(906, c.getStartLine());
		assertEquals(9, c.getStartColumn());
		assertEquals(916, c.getEndLine());
		assertEquals(9, c.getEndColumn());

		assertEquals(6, cs.getOldId());
		assertEquals(2, ol.size());

		c = ol.get(0);
		assertEquals(13, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "helper\\ProjectHelperImpl.java");
		assertEquals(sf, c.getFile());
		assertEquals("ProjectHelperImpl.TaskHandler.startElement(String name,AttributeList attrs)", c.getMethodName());
		assertEquals(771, c.getStartLine());
		assertEquals(9, c.getStartColumn());
		assertEquals(779, c.getEndLine());
		assertEquals(9, c.getEndColumn());

		c = ol.get(1);
		assertEquals(14, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "helper\\ProjectHelperImpl.java");
		assertEquals(sf, c.getFile());
		assertEquals("ProjectHelperImpl.NestedElementHandler.startElement(String name,AttributeList attrs)", c.getMethodName());
		assertEquals(906, c.getStartLine());
		assertEquals(9, c.getStartColumn());
		assertEquals(916, c.getEndLine());
		assertEquals(9, c.getEndColumn());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(7);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(7, cs.getId());
		assertEquals(2, nl.size());

		c = nl.get(0);
		assertEquals(15, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "IntrospectionHelper.java");
		assertEquals(sf, c.getFile());
		assertEquals("IntrospectionHelper.createElement(Project project,Object parent,String elementName)", c.getMethodName());
		assertEquals(609, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(627, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = nl.get(1);
		assertEquals(16, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "IntrospectionHelper.java");
		assertEquals(sf, c.getFile());
		assertEquals("IntrospectionHelper.storeElement(Project project,Object parent,Object child,String elementName)", c.getMethodName());
		assertEquals(785, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(805, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		assertEquals(7, cs.getOldId());
		assertEquals(2, ol.size());

		c = ol.get(0);
		assertEquals(15, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "IntrospectionHelper.java");
		assertEquals(sf, c.getFile());
		assertEquals("IntrospectionHelper.createElement(Project project,Object parent,String elementName)", c.getMethodName());
		assertEquals(609, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(627, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = ol.get(1);
		assertEquals(16, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "IntrospectionHelper.java");
		assertEquals(sf, c.getFile());
		assertEquals("IntrospectionHelper.storeElement(Project project,Object parent,Object child,String elementName)", c.getMethodName());
		assertEquals(785, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(805, c.getEndLine());
		assertEquals(5, c.getEndColumn());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(8);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(8, cs.getId());
		assertEquals(2, nl.size());

		c = nl.get(0);
		assertEquals(17, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "listener\\BigProjectLogger.java");
		assertEquals(sf, c.getFile());
		assertEquals("BigProjectLogger.subBuildStarted(BuildEvent event)", c.getMethodName());
		assertEquals(122, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(137, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = nl.get(1);
		assertEquals(18, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "listener\\BigProjectLogger.java");
		assertEquals(sf, c.getFile());
		assertEquals("BigProjectLogger.subBuildFinished(BuildEvent event)", c.getMethodName());
		assertEquals(156, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(165, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		assertEquals(8, cs.getOldId());
		assertEquals(2, ol.size());

		c = ol.get(0);
		assertEquals(17, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "listener\\BigProjectLogger.java");
		assertEquals(sf, c.getFile());
		assertEquals("BigProjectLogger.subBuildStarted(BuildEvent event)", c.getMethodName());
		assertEquals(122, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(137, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = ol.get(1);
		assertEquals(18, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "listener\\BigProjectLogger.java");
		assertEquals(sf, c.getFile());
		assertEquals("BigProjectLogger.subBuildFinished(BuildEvent event)", c.getMethodName());
		assertEquals(156, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(165, c.getEndLine());
		assertEquals(5, c.getEndColumn());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(9);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(9, cs.getId());
		assertEquals(2, nl.size());

		c = nl.get(0);
		assertEquals(19, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "listener\\CommonsLoggingListener.java");
		assertEquals(sf, c.getFile());
		assertEquals("CommonsLoggingListener.taskStarted(BuildEvent event)", c.getMethodName());
		assertEquals(167, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(183, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = nl.get(1);
		assertEquals(20, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "listener\\CommonsLoggingListener.java");
		assertEquals(sf, c.getFile());
		assertEquals("CommonsLoggingListener.taskFinished(BuildEvent event)", c.getMethodName());
		assertEquals(189, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(211, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		assertEquals(9, cs.getOldId());
		assertEquals(2, ol.size());

		c = ol.get(0);
		assertEquals(19, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "listener\\CommonsLoggingListener.java");
		assertEquals(sf, c.getFile());
		assertEquals("CommonsLoggingListener.taskStarted(BuildEvent event)", c.getMethodName());
		assertEquals(167, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(183, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = ol.get(1);
		assertEquals(20, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "listener\\CommonsLoggingListener.java");
		assertEquals(sf, c.getFile());
		assertEquals("CommonsLoggingListener.taskFinished(BuildEvent event)", c.getMethodName());
		assertEquals(189, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(211, c.getEndLine());
		assertEquals(5, c.getEndColumn());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(10);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(10, cs.getId());
		assertEquals(2, nl.size());

		c = nl.get(0);
		assertEquals(21, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "taskdefs\\compilers\\DefaultCompilerAdapter.java");
		assertEquals(sf, c.getFile());
		assertEquals("DefaultCompilerAdapter.logAndAddFilesToCompile(Commandline cmd)", c.getMethodName());
		assertEquals(394, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(415, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = nl.get(1);
		assertEquals(22, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "taskdefs\\rmic\\DefaultRmicAdapter.java");
		assertEquals(sf, c.getFile());
		assertEquals("DefaultRmicAdapter.logAndAddFilesToCompile(Commandline cmd)", c.getMethodName());
		assertEquals(329, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(350, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		assertEquals(-1, cs.getOldId());
		assertEquals(0, ol.size());
		//----------------------------------------

		//----------------------------------------
		cs = csList.get(11);
		nl = cs.getNewCloneList();
		ol = cs.getOldCloneList();

		assertEquals(-1, cs.getId());
		assertEquals(0, nl.size());

		assertEquals(10, cs.getOldId());
		assertEquals(2, ol.size());

		c = ol.get(0);
		assertEquals(21, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "PropertyHelper.java");
		assertEquals(sf, c.getFile());
		assertEquals("PropertyHelper.setProperty(String name,Object value,boolean verbose)", c.getMethodName());
		assertEquals(632, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(660, c.getEndLine());
		assertEquals(5, c.getEndColumn());

		c = ol.get(1);
		assertEquals(22, c.getId());
		assertEquals(cs, c.getCloneSet());
		sf = Project.getFileObj(p.getFileList(), "PropertyHelper.java");
		assertEquals(sf, c.getFile());
		assertEquals("PropertyHelper.setNewProperty(String name,Object value)", c.getMethodName());
		assertEquals(693, c.getStartLine());
		assertEquals(5, c.getStartColumn());
		assertEquals(713, c.getEndLine());
		assertEquals(5, c.getEndColumn());
		//----------------------------------------
	}

	/**
	 * t@C̃Rs[
	 * @param dist
	 * @param source
	 */
	private void copyFile(String dist, String source) throws Exception {
		FileInputStream fis = new FileInputStream(source);
		FileOutputStream fos = new FileOutputStream(dist);

		byte buf[] = new byte[256];
		int len;
		while((len = fis.read(buf)) != -1) {
			fos.write(buf, 0, len);
		}

		fos.flush();
		fos.close();
		fis.close();

		System.err.println("Copy " + source + " -> " + dist);
	}


	/*
	 * :
	 *
	 * Ԃl:
	 *    true: 
	 *   false: s
	 *
	 * O:
	 *
	 * Ȃ, s͏oȂ͂.
	 */

	/**
	 * s(java)
	 */
	@Test
	public void executeTest01() throws Exception {
		Logger.init();

		Project p = new Project();
		p.setLang("java");
		File newdir = new File("testfile/Application/ant");
		File olddir = new File("testfile/Application/ant");
		p.setNewDir(newdir.getAbsolutePath());
		p.setOldDir(olddir.getAbsolutePath());

		File file = new File("file");
		if(file.listFiles().length > 0) {
			File[] list = file.listFiles();
			for(int i = 0; i < list.length; i++) {
				list[i].delete();
			}
		}

		CloneDetectorController cdc = new CloneDetectorController(p);

		boolean result = cdc.execute();

		assertTrue(result);

		File newText = new File(Def.RESULT_TXT);
		assertTrue(newText.exists());

		File newCSV = new File(Def.RESULT_CSV);
		assertTrue(newCSV.exists());

		File oldText = new File(Def.RESULT_TXT_OLD);
		assertTrue(oldText.exists());

		File oldCSV = new File(Def.RESULT_CSV_OLD);
		assertTrue(oldCSV.exists());

		File correctText = new File("testfile\\Application\\ant.txt");
		File correctCSV = new File("testfile\\Application\\ant.csv");

		BufferedReader br = new BufferedReader(new FileReader(correctText));
		ArrayList<String> corText = new ArrayList<String>();
		String line;
		while((line = br.readLine()) != null) {
			corText.add(line);
		}
		br.close();

		br = new BufferedReader(new FileReader(correctCSV));
		ArrayList<String> corCSV = new ArrayList<String>();
		while((line = br.readLine()) != null) {
			corCSV.add(line);
		}
		br.close();

		ArrayList<String> testList;

		testList = new ArrayList<String>();
		br = new BufferedReader(new FileReader(newText));
		while((line = br.readLine()) != null) {
			testList.add(line);
		}
		br.close();
		assertEquals(corText.size(), testList.size());
		for(int i = 0; i < corText.size(); i++) {
			assertEquals(corText.get(i), testList.get(i));
		}

		testList = new ArrayList<String>();
		br = new BufferedReader(new FileReader(newCSV));
		while((line = br.readLine()) != null) {
			testList.add(line);
		}
		br.close();
		assertEquals(corCSV.size(), testList.size());
		for(int i = 0; i < corCSV.size(); i++) {
			assertEquals(corCSV.get(i), testList.get(i));
		}

		testList = new ArrayList<String>();
		br = new BufferedReader(new FileReader(oldText));
		while((line = br.readLine()) != null) {
			testList.add(line);
		}
		br.close();
		assertEquals(corText.size(), testList.size());
		for(int i = 0; i < corText.size(); i++) {
			assertEquals(corText.get(i), testList.get(i));
		}

		testList = new ArrayList<String>();
		br = new BufferedReader(new FileReader(oldCSV));
		while((line = br.readLine()) != null) {
			testList.add(line);
		}
		br.close();
		assertEquals(corCSV.size(), testList.size());
		for(int i = 0; i < corCSV.size(); i++) {
			assertEquals(corCSV.get(i), testList.get(i));
		}
	}

	/**
	 * s(C)
	 */
	@Test
	public void executeTest02() throws Exception {
		Logger.init();

		Project p = new Project();
		p.setLang("c");
		File newdir = new File("testfile/Application/toppers");
		File olddir = new File("testfile/Application/toppers");
		p.setNewDir(newdir.getAbsolutePath());
		p.setOldDir(olddir.getAbsolutePath());

		File file = new File("file");
		if(file.listFiles().length > 0) {
			File[] list = file.listFiles();
			for(int i = 0; i < list.length; i++) {
				list[i].delete();
			}
		}

		CloneDetectorController cdc = new CloneDetectorController(p);

		boolean result = cdc.execute();

		assertTrue(result);

		File newText = new File(Def.RESULT_TXT);
		assertTrue(newText.exists());

		File newCSV = new File(Def.RESULT_CSV);
		assertTrue(newCSV.exists());

		File oldText = new File(Def.RESULT_TXT_OLD);
		assertTrue(oldText.exists());

		File oldCSV = new File(Def.RESULT_CSV_OLD);
		assertTrue(oldCSV.exists());

		File correctText = new File("testfile\\Application\\toppers.txt");
		File correctCSV = new File("testfile\\Application\\toppers.csv");

		BufferedReader br = new BufferedReader(new FileReader(correctText));
		ArrayList<String> corText = new ArrayList<String>();
		String line;
		while((line = br.readLine()) != null) {
			corText.add(line);
		}
		br.close();

		br = new BufferedReader(new FileReader(correctCSV));
		ArrayList<String> corCSV = new ArrayList<String>();
		while((line = br.readLine()) != null) {
			corCSV.add(line);
		}
		br.close();

		ArrayList<String> testList;

		testList = new ArrayList<String>();
		br = new BufferedReader(new FileReader(newText));
		while((line = br.readLine()) != null) {
			testList.add(line);
		}
		br.close();
		assertEquals(corText.size(), testList.size());
		for(int i = 0; i < corText.size(); i++) {
			assertEquals(corText.get(i), testList.get(i));
		}

		testList = new ArrayList<String>();
		br = new BufferedReader(new FileReader(newCSV));
		while((line = br.readLine()) != null) {
			testList.add(line);
		}
		br.close();
		assertEquals(corCSV.size(), testList.size());
		for(int i = 0; i < corCSV.size(); i++) {
			assertEquals(corCSV.get(i), testList.get(i));
		}

		testList = new ArrayList<String>();
		br = new BufferedReader(new FileReader(oldText));
		while((line = br.readLine()) != null) {
			testList.add(line);
		}
		br.close();
		assertEquals(corText.size(), testList.size());
		for(int i = 0; i < corText.size(); i++) {
			assertEquals(corText.get(i), testList.get(i));
		}

		testList = new ArrayList<String>();
		br = new BufferedReader(new FileReader(oldCSV));
		while((line = br.readLine()) != null) {
			testList.add(line);
		}
		br.close();
		assertEquals(corCSV.size(), testList.size());
		for(int i = 0; i < corCSV.size(); i++) {
			assertEquals(corCSV.get(i), testList.get(i));
		}
	}
}
