


void row_dealloc(Row* self)
{
    Py_XDECREF(self->data);
    Py_XDECREF(self->description);

    self->ob_type->tp_free((PyObject*)self);
}

int row_init(Row* self, PyObject* args, PyObject* kwargs)
{
    PyObject* data;
    Cursor* cursor;

    self->data = 0;
    self->description = 0;

    if (!PyArg_ParseTuple(args, "OO", &cursor, &data)) {
        return -1;
    }

    if (!PyObject_IsInstance((PyObject*)cursor, (PyObject*)&CursorType)) {
        PyErr_SetString(PyExc_TypeError, "instance of cursor required for first argument");
        return -1;
    }

    if (!PyTuple_Check(data)) {
        PyErr_SetString(PyExc_TypeError, "tuple required for second argument");
        return -1;
    }

    Py_INCREF(data);
    self->data = data;

    Py_INCREF(cursor->description);
    self->description = cursor->description;

    return 0;
}

PyObject* row_subscript(Row* self, PyObject* idx)
{
    long _idx;
    char* key;
    int nitems, i;
    char* compare_key;

    char* p1;
    char* p2;

    PyObject* item;

    if (PyInt_Check(idx)) {
        _idx = PyInt_AsLong(idx);
        item = PyTuple_GetItem(self->data, _idx);
        Py_XINCREF(item);
        return item;
    } else if (PyLong_Check(idx)) {
        _idx = PyLong_AsLong(idx);
        item = PyTuple_GetItem(self->data, _idx);
        Py_XINCREF(item);
        return item;
    } else if (PyString_Check(idx)) {
        key = PyString_AsString(idx);

        nitems = PyTuple_Size(self->description);

        for (i = 0; i < nitems; i++) {
            compare_key = PyString_AsString(PyTuple_GET_ITEM(PyTuple_GET_ITEM(self->description, i), 0));
            if (!compare_key) {
                return NULL;
            }

            p1 = key;
            p2 = compare_key;

            while (1) {
                if ((*p1 == (char)0) || (*p2 == (char)0)) {
                    break;
                }

                if ((*p1 | 0x20) != (*p2 | 0x20)) {
                    break;
                }

                p1++;
                p2++;
            }

            if ((*p1 == (char)0) && (*p2 == (char)0)) {
                
                item = PyTuple_GetItem(self->data, i);
                Py_INCREF(item);
                return item;
            }

        }

        PyErr_SetString(PyExc_IndexError, "No item with that key");
        return NULL;
    } else if (PySlice_Check(idx)) {
        PyErr_SetString(PyExc_ValueError, "slices not implemented, yet");
        return NULL;
    } else {
        PyErr_SetString(PyExc_IndexError, "Index must be int or string");
        return NULL;
    }
}

Py_ssize_t row_length(Row* self, PyObject* args, PyObject* kwargs)
{
    return PyTuple_GET_SIZE(self->data);
}

static int row_print(Row* self, FILE *fp, int flags)
{
    return (&PyTuple_Type)->tp_print(self->data, fp, flags);
}


PyMappingMethods row_as_mapping = {
    
    
    
};


PyTypeObject RowType = {
        PyObject_HEAD_INIT(NULL)
        0,                                              
        MODULE_NAME ".Row",                             
        sizeof(Row),                                    
        0,                                              
        (destructor)row_dealloc,                        
        (printfunc)row_print,                           
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE,         
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        0,                                              
        (initproc)row_init,                             
        0,                                              
        0,                                              
        0                                               
};

extern int row_setup_types(void)
{
    RowType.tp_new = PyType_GenericNew;
    RowType.tp_as_mapping = &row_as_mapping;
    return PyType_Ready(&RowType);
}
